/*******************************************************************************
 * Chips & Technologies chipset driver
 *******************************************************************************
 *
   Copyright (C) 1997 Stephan Meyer <Stephan.Meyer@pobox.com>
 *
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************************
 * 
 * This code was derived from the following sources of information:
 * [1] the technical specifications available from www.chips.com
 * [2] Xfree86 Chips driver
 * [3] svgalib 1.2.11 Chips driver
 *
 * Many thanks go to David Bateman <dbateman@ee.uts.edu.au> et al. 
**
**	$Log: 655xx.c,v $
**	Revision 1.8  1999/06/21 23:44:58  marcus
**	Don't include any Linux-specific stuff in KGI drivers.
**	
**	Revision 1.7  1998/11/19 02:43:03  marcus
**	License change
**	
**	Revision 1.6  1998/11/01 21:01:43  brian
**	Changed cmd to *cmd in chipset_check_mode
**	Removed DEFINE_SUBSYSTEM() from module.h (yay.)
**	Some namespace/typedef cleanup -- see mailing list and comment please
**	Squashed a few bugs due to drivers not synced with graphics->accel rename
**	
**	Revision 1.5  1998/10/25 18:09:36  marcus
**	More namespace cleanups
**	
**	Revision 1.4  1998/10/25 02:04:26  marcus
**	Namespace cleanup to make KGI/LibGGI interaction possible
**	
**	Revision 1.3  1998/10/04 03:16:24  brian
**	Added private data hook to KGI_COMMON -- clean and recompile kgicon!
**	
**	Revision 1.2  1998/08/19 20:47:09  marcus
**	Changed ggi/*.h includes to kgi/*.h includes
**	Fixed most of the remaining graphics -> accel changes
**	
**	Revision 1.1  1998/07/07 07:12:34  taylorj
**	*** empty log message ***
**	
**	Revision 1.4  1997/12/14 21:55:31  becka
**	uint/sint namespace fixed ... please test ...
**	
**	Revision 1.3  1997/12/03 16:05:53  seeger
**	* small include bugs fixed.
**	
**	Revision 1.2  1997/11/08 15:12:22  becka
**	Namespace cleanup ... probably broke all and everything ...
**	
**	Revision 1.1  1997/10/22 17:31:21  seeger
**	Initial C&T driver checkin
**	
**
*/

/* FIXME 6555x are not yet fully supported! */

#include <kgi/maintainers.h>
#define	MAINTAINER	Stephan_Meyer
#define DRIVER_NAME	"Chips & Technologies chipset driver"
#define DRIVER_REV	"$Revision: 1.8 $"

#include <kgi/system.h>
#include <kgi/module.h>

#include "Chips/chips.h"

#include "IBM/vga.h"
#include "IBM/vga_io.inc"
#include "chips_text16.inc"

static struct kgi_chipset chipinfo =
{
	"Chips & Technologies",	/* Manufacturer				*/
	"655xx",		/* Model (detect)			*/
	GM_TEXT16 | GM_8BIT |	/* Available modes			*/
	GM_16BIT | GM_24BIT,
	0,			/* Flags				*/
	{ 0xFFFF, 0xFFFF },	/* Maximum resolution (autodetect)	*/
	NULL,			/* Private data hook			*/
	0,			/* ramsize (autodetect)			*/
	{ 0, 80000000 }		/* DCLK limits		FIXME		*/
};

static struct io_region chips_io =
{
	IO_NULL,
	0x3C0,
	0x20,
	0x3FF,
	"Chips 655xx IO"
};

static struct mem_region chips_mem_lin =
{
	MEM_NULL,
	0x0,			/* linear fb (autodetect) */
	0x0,			/* memory size (autodetect) */
	-1,			/* decode all */
	"Chips 655xx MEMlin"
};

static struct mem_region chips_mem_txt =
{
	MEM_NULL,
	0xA0000,
	0x10000,
	-1,
	"Chips 655xx MEMtxt"
};

extern struct kgi_ramdac ramdac;

static kgiu16 chips_devids[] =
{
	PCI_DEVICE_ID_CT_65545,
	PCI_DEVICE_ID_CT_65548,
	PCI_DEVICE_ID_CT_65550,
	PCI_DEVICE_ID_CT_65554,
	0
};

kgi_sint	chips_model;
kgiu16		chips_iobase;

/* register values saved at init */
ChipsRegs	chips_resetregs;

kgi_modereq	default_kgi_modereq;
struct kgi_mode	default_kgi_mode;

/* preset panel timings */
kgi_sint	chips_hdisplay;
kgi_sint	chips_hsyncstart;
kgi_sint	chips_hsyncend;
kgi_sint	chips_htotal;
kgi_sint	chips_vdisplay;
kgi_sint 	chips_vsyncstart;
kgi_sint	chips_vtotal;

static kgiu8 chips_extensionregs[] =	/* extension regs to be saved */
	{0x02,0x03,0x04,0x06,0x07,0x0B,0x0D,0x0F,0x17,0x19,
	 0x1A,0x1B,0x1C,0x1E,0x21,0x22,0x23,0x28,0x2C,0x2D,
	 0x2F,0x33,0x40,0x50,0x51,0x52,0x54,0x55,0x56,0x57,
	 0x58,0x5A,0x64,0x65,0x66,0x67,0x68,0x6F,0};

/* enable access to chip registers */
static void chips_unlock(void)
{
	kgiu8 b;

	DEBUG1("unlock: enter\n");
	
	io_out16(0x0010,0x3D6);
	io_out16(0x0015,0x3D6);
	io_out8(0x14,0x3D6);
	b=io_in8(0x3D7);
	io_out8(b & ~0x20,0x3D7);	
	outb(0x11,chips_iobase+4);
	b=io_in8(chips_iobase+5);
	io_out8(b & 0x7F,chips_iobase+5);

	DEBUG1("unlock: exit\n");
}

/* save the current register values */
static void chips_getregs(ChipsRegs *regs)
{
	kgiu8 i,a;

	DEBUG1("getregs: enter\n");

	chips_unlock();

	regs->miscoutput=io_in8(0x3CC);

	for (i=0;i<CHIPS_CRTC_COUNT;i++) {
		io_out8(i,chips_iobase+4);
		regs->crtc[i]=io_in8(chips_iobase+5);
	}

	for (i=0;i<CHIPS_ATC_COUNT;i++) {
		a=io_in8(0x3DA);
		io_out8(i,0x3C0);
		regs->atc[i]=io_in8(0x3C1);
	}

	for (i=0;i<CHIPS_GRAPHICS_COUNT;i++) {
		io_out8(i,0x3CE);
		regs->graphics[i]=io_in8(0x3CF);
	}

	for (i=0;i<CHIPS_SEQUENCER_COUNT;i++) {
		io_out8(i,0x3C4);
		regs->sequencer[i]=io_in8(0x3C5);
	}

	for (i=0;chips_extensionregs[i];i++) {
		io_out8(chips_extensionregs[i],0x3D6);
		regs->extension[chips_extensionregs[i]]=io_in8(0x3D7);
	}

	/* save clock */
	chips_calcclock(ramdac.clock[KGIGT_TEXT16].max,regs);
	
	DEBUG1("getregs: exit\n");
}

/* set the registers from the saved values */
static void chips_setregs(ChipsRegs *regs)
{
	kgiu8 i,a;

	DEBUG1("setregs: enter\n");

	chips_unlock();

	io_out8(regs->miscoutput,0x3C2);

	for (i=0;i<CHIPS_CRTC_COUNT;i++) {
		io_out8(i,chips_iobase+4);
		io_out8(regs->crtc[i],chips_iobase+5);
	}

	for (i=0;i<CHIPS_ATC_COUNT;i++) {
		a=io_in8(0x3DA);
		io_out8(i,0x3C0);
		io_out8(regs->atc[i],0x3C1);
	}

	for (i=0;i<CHIPS_GRAPHICS_COUNT;i++) {
		io_out8(i,0x3CE);
		io_out8(regs->graphics[i],0x3CF);
	}

	io_out8(0x00,0x3C4);
	io_out8(0x01,0x3C5);
	for (i=0;i<CHIPS_SEQUENCER_COUNT;i++) {
		io_out8(i,0x3C4);
		io_out8(regs->sequencer[i],0x3C5);
	}
	io_out8(0x00,0x3C4);
	io_out8(0x03,0x3C5);

	for (i=0;chips_extensionregs[i];i++) {
		io_out8(chips_extensionregs[i],0x3D6);
		io_out8(regs->extension[chips_extensionregs[i]],0x3D7);
	}

	/* restore clock */
	io_out8(0x54,0x3D6);
	a=io_in8(0x3D7);
	io_out8((a & 0xF3) | 0x04,0x3D7);
	io_out8(0x33,0x3D6);
	a=io_in8(0x3D7);
	io_out8(a & ~0x20,0x3D7);
	io_out8(0x30,0x3D6);
	io_out8(regs->vclock[0],0x3D7);
	io_out8(0x31,0x3D6);
	io_out8(regs->vclock[1],0x3D7);
	io_out8(0x32,0x3D6);
	io_out8(regs->vclock[2],0x3D7);
	io_out8(0x33,0x3D6);
	a=io_in8(0x3D7);
	io_out8((regs->extension[0x33] & 0x20) | (a & ~0x20),0x3D7);
	io_out8(0x54,0x3D6);
	a=io_in8(0x3D7);
	io_out8((regs->extension[0x54] & 0x0C) | (a & ~0x0C),0x3D7);
	
	DEBUG1("setregs: exit\n");
}

static void chips_setup_dpy(struct kgi_display *dpy,struct kgi_mode *mode)
{
	switch (mode->request.graphtype) {
	case KGIGT_TEXT16 :
		dpy->set_font=text16_set_font;
                dpy->set_origin=text16_set_origin;
		dpy->set_split=text16_set_split;
		dpy->set_frame=text16_set_frame;
		dpy->show_cursor=text16_show_cursor;
		dpy->hide_cursor=text16_hide_cursor;
		dpy->undo_cursor=text16_undo_cursor;
		dpy->show_pointer=text16_show_pointer;
		dpy->hide_pointer=text16_hide_pointer;
		dpy->undo_pointer=text16_undo_pointer;
		dpy->fb.dpy=dpy;
		dpy->fb.size.x = mode->request.virt.x/mode->request.text.x;
		dpy->fb.size.y = mode->request.virt.y/mode->request.text.y;
		dpy->fb.bpp    = MAP_16_BPP;
		dpy->fb.pbuf   = chips_mem_txt.base;
		dpy->fb.zbuf   = NULL;
		dpy->fb.ops    = &text16_color_io;
		dpy->driver_data=text16_data_realloc(dpy->driver_data);
		if (dpy->driver_data==NULL) {

			INTERNAL_ERROR;
		}
		break;
                default: INTERNAL_ERROR; break;
        }
        memcpy(&(dpy->mode),mode,sizeof(struct kgi_mode));
}                                                                                                         

static void chips_setup_mode(struct kgi_mode *mode)
{
	switch (mode->request.graphtype) {
	case KGIGT_TEXT16 :
		mode->fb=(mem_vaddr *)chips_mem_txt.base;
		mode->fb_size=chips_mem_txt.size;
		mode->fsize=(mode->request.virt.x/mode->request.text.x)*
			(mode->request.virt.y/mode->request.text.y)*2;
		mode->mem=&text16_color_mem;
		mode->io=&text16_color_io;
		mode->mmio=NULL;
		break;
	default:
		INTERNAL_ERROR;
		break;
	}
}

/* fill dpy with info about the preset mode */
/* for now, this driver just fills in the 80x25 text mode data :) */
static kgi_uint chips_get_reset_mode(struct kgi_display *dpy)
{
	struct kgi_preset t;
	struct kgi_graphic g;
	
	t.x.width=640;
	t.y.width=400;
	g.io_res=g.io_buf=&t;
	kgim_monitor_command(&g,MONITOR_GETPRESETMODE,&t);
	
	default_kgi_modereq.frames=1;
	default_kgi_modereq.visible.x=default_kgi_modereq.virt.x=640;
	default_kgi_modereq.visible.y=default_kgi_modereq.virt.y=400;
	default_kgi_modereq.size.x=default_kgi_modereq.size.y=0;
	default_kgi_modereq.graphtype=KGIGT_TEXT16;
	default_kgi_modereq.text.x=8;
	default_kgi_modereq.text.y=16;
	if (dpy->defmode==NULL) dpy->defmode=&default_kgi_modereq; else
			(*dpy->defmode)=default_kgi_modereq;
	dpy->mode.request=default_kgi_modereq;
	dpy->mode.x=t.x;
	dpy->mode.y=t.y;
	dpy->mode.magnify.x=dpy->mode.magnify.y=1;
	dpy->mode.dclk=25175000;
	dpy->mode.clk.mul=dpy->mode.clk.div=dpy->mode.lclk.mul=dpy->mode.lclk.div=1;
	dpy->mode.chipset=0;
	default_kgi_mode=dpy->mode;

	chips_setup_mode(&default_kgi_mode);
	chips_setup_dpy(dpy,&(default_kgi_mode));
	
	return EOK;
}
	
/* do initial setup and probing */
static void chips_setup()
{
	kgi_sint t;
	
	t=chips_resetregs.extension[0x17];
	chips_hdisplay=((((t & 0x10) >> 4) << 8) + 
			chips_resetregs.extension[0x1C] + 1) << 3;
	chips_hsyncstart=((((t & 0x04) >> 2) << 8) +
			chips_resetregs.extension[0x19] + 1) << 3;
	chips_hsyncend=((((t & 0x08) >> 3) << 5) +
			(chips_resetregs.extension[0x1A] & 0x1F) -
			(chips_resetregs.extension[0x19] & 0x1F)) << 3;
	chips_htotal=(((t & 0x01) << 8) +
			chips_resetregs.extension[0x1B] + 5) << 3;
			
	t=chips_resetregs.extension[0x65];
	chips_vdisplay=(((t & 0x40) >> 6) << 9) + (((t & 0x02) >> 1) << 8) +
			chips_resetregs.extension[0x68] + 1;
	chips_vsyncstart=(((t & 0x80) >> 7) << 9) + (((t & 0x04) >> 2) << 8) +
			chips_resetregs.extension[0x66] + 1;
	chips_vtotal=(((t & 0x20) >> 4) << 9) + ((t & 0x01) << 8) +
			chips_resetregs.extension[0x64] + 2;

	DEBUG1("hdisplay: %i  hsyncstart: %i  hsyncend: %i  htotal: %i\n",
			chips_hdisplay,chips_hsyncstart,chips_hsyncend,chips_htotal);
	DEBUG1("vdisplay: %i  vsyncstart %i  vtotal: %i\n",
			chips_vdisplay,chips_vsyncstart,chips_vtotal);

	text16_data_init();
}

kgi_sint kgim_chipset_init(struct kgi_card *card, struct kgi_display *dpy) {
	kgi_sint i,err;
	kgiu8 bus,id;
	kgiu32 base;

	NOTICE(DRIVER_NAME DRIVER_REV);

	/* only PCI configurations are supported */
	if (!pcibios_present()) {
		ERROR("No PCI BIOS detected!");
		return -E(CHIPSET,UNKNOWN);
	}
	
	for (i=0; chips_devids[i]!=0; i++) {

		if (pcibios_find_device(PCI_VENDOR_ID_CT, chips_devids[i],0,
                        &bus,&id)==PCIBIOS_SUCCESSFUL) {

			break;
		}
	}

	if (!chips_devids[i]) {

		ERROR("No Chips & Technologies 655xx chip detected!");
		return -E(CHIPSET,UNKNOWN);
	}

	switch (i) {

	case 0 :
		strcpy(&chipinfo.model,"65545");
		chips_model=CHIPS_65545;
		break;

	case 1 :
		strcpy(&chipinfo.model,"65548");
		chips_model=CHIPS_65548;
		break;
	case 2 :
		strcpy(&chipinfo.model,"65550");
		chips_model=CHIPS_65550;
		break;
	case 3 :
		strcpy(&chipinfo.model,"65554");
		chips_model=CHIPS_65554;
		break;
	}
		
	if ((err = io_check_region(&chips_io)) != EOK) {

		return err;
	}
	io_claim_region(&chips_io);

	chips_iobase=(io_in8(0x3CC) & 1) ? 0x3D0 : 0x3B0;

	/* read framebuffer base address */
	pcibios_read_config_dword(bus,id,0x10,&base);
	chips_mem_lin.base=base & 0xFF000000;

	chips_getregs(&chips_resetregs);
	
	switch (chips_resetregs.crtc[0x0F] & 3) {

	case 0:
		chips_mem_lin.size=256*1024;
		break;

	case 1:
		chips_mem_lin.size=512*1024;
		break;

	case 2:
	case 3:
		chips_mem_lin.size=1024*1024;
		break;
	}
	chipinfo.ramsize=chips_mem_lin.size;

	if (((err=mem_check_region(&chips_mem_lin))!=EOK) ||
	    ((err=mem_check_region(&chips_mem_txt))!=EOK)) {

		io_free_region(&chips_io);
		return err;
	}
	mem_claim_region(&chips_mem_lin);
	mem_claim_region(&chips_mem_txt);

	DEBUG1("chipset found: %s\n",chipinfo.model);
	DEBUG1("iobase: %x  membase: %x  memsize: %i\n", \
		chips_iobase, chips_mem_lin.base, chips_mem_lin.size);

	chips_setup();

	chips_get_reset_mode(dpy);
	strcpy(card->manufact,chipinfo.manufact);
	strcpy(card->model,chipinfo.model);
	card->modes=chipinfo.modes;
	card->flags=chipinfo.flags;
	card->maxdots=chipinfo.maxdots;
	card->chipset=&chipinfo;
	card->preset=default_kgi_mode;

	return EOK;
}

void kgim_chipset_done(struct kgi_display *dpy)
{
	mem_free_region(&chips_mem_txt);
	mem_free_region(&chips_mem_lin);
	io_free_region(&chips_io);
	NOTICE(DRIVER_NAME " removed");
}

kgi_sint kgim_chipset_check_mode(struct kgi_display *dpy,struct kgi_mode *mode,
	enum kgi_tm_cmd *cmd)
{
	kgi_sint bpp;
	kgi_sint textx=mode->request.text.x, texty=mode->request.text.y;

	DEBUG1("kgim_chipset_check_mode: enter\n");

	switch (mode->request.graphtype) {

	case KGIGT_TEXT16:
	 	break;

	case KGIGT_8BIT:
		bpp=8;
		break;

	case KGIGT_16BIT:
		bpp=16;
		break;

	case KGIGT_24BIT:
		bpp=24;
		break;

	default:
		return -E(CHIPSET,FAILED);
	}
	
#define XVIRTUAL (mode->request.virt.x)
#define YVIRTUAL (mode->request.virt.y)
#define XVISIBLE (mode->request.visible.x)
#define YVISIBLE (mode->request.visible.y)
	
	switch (*cmd) {
	case CMD_PROPOSE :
		if (mode->request.graphtype==KGIGT_TEXT16) {
			mode->magnify.x=mode->magnify.y=1;
			if ((textx<8) || (textx>9)) return -EINVAL;
			XVISIBLE-=XVISIBLE%textx;
			YVISIBLE-=YVISIBLE%texty;
			if (XVIRTUAL==0 && YVIRTUAL==0) return -EINVAL;
			if (XVIRTUAL==0)
			  XVIRTUAL=chips_mem_txt.size/((YVIRTUAL/texty)*2)*textx;
			if (YVIRTUAL==0)
			  YVIRTUAL=chips_mem_txt.size/((XVIRTUAL/textx)*2)*texty;
			if ((2*(XVIRTUAL/textx)*(YVIRTUAL/texty))>chips_mem_txt.size)
			  return -ENOMEM;
		} else {
			XVISIBLE&=~0x07;
			mode->magnify.x=mode->magnify.y=1;
			if (XVIRTUAL==0 && YVIRTUAL==0) return -EINVAL;
			if (XVIRTUAL==0)
			  XVIRTUAL=(chips_mem_lin.size/bpp*8)/YVIRTUAL;
				if (YVIRTUAL==0)
			  YVIRTUAL=(chips_mem_lin.size/bpp*8)/XVIRTUAL;
			if ((XVIRTUAL*YVIRTUAL*bpp/8)>chips_mem_lin.size) {
				if ((XVISIBLE*YVISIBLE*bpp/8)>chips_mem_lin.size)
				  return -ENOMEM;
				XVIRTUAL=XVISIBLE;
				YVIRTUAL=YVISIBLE;
			}
			XVISIBLE&=0xFFFFFFF8;
			YVISIBLE&=0xFFFFFFF8;
		}  
		mode->x.width=mode->request.visible.x*mode->magnify.x;
		mode->y.width=mode->request.visible.x*mode->magnify.y;
		mode->clk.div=mode->lclk.div=1;
		return EOK;

	case CMD_LOWER :
		return EOK;

	case CMD_RAISE :
		return EOK;

	case CMD_CHECK :
		if (mode->lclk.div!=1) return -E(CHIPSET,MODE_CHECK);
		if (mode->request.graphtype==KGIGT_TEXT16) {
			mode->flags|=FL_CHIP_COLORTEXT;
			if ((mode->magnify.x!=1) ||
			    (mode->magnify.y!=1) ||
				    (textx<8) || (textx>9) ||
			    (texty<1) || (texty>16) ||
			    ((2*(XVIRTUAL/textx)*(YVIRTUAL/texty))>chips_mem_txt.size) ||
			    (mode->dclk>chipinfo.dclk.max) ||
			    (mode->dclk<chipinfo.dclk.min))
				return -E(CHIPSET,MODE_CHECK);
		} else {
			if (((XVIRTUAL*YVIRTUAL*bpp/8)>chips_mem_lin.size) ||
			    (mode->magnify.x>2) ||
			    (mode->magnify.y>2) ||
			    (mode->dclk*mode->clk.mul/mode->clk.div<chipinfo.dclk.min) ||
			    (mode->dclk*mode->clk.mul/mode->clk.div>chipinfo.dclk.max))
				return -E(CHIPSET,MODE_CHECK);
		}		
		chips_setup_mode(mode);
		return EOK;
	default:
		INTERNAL_ERROR;
		return -E(CHIPSET,FAILED);
	}

#undef XVIRTUAL
#undef YVIRTUAL
#undef XVISIBLE
#undef YVISIBLE
	
	DEBUG1("kgim_chipset_check_mode: exit\n");						
}

void kgim_chipset_set_mode(struct kgi_display *dpy,struct kgi_mode *mode)
{
	DEBUG1("kgim_chipset_set_mode: void\n");
}

kgi_sint kgim_chipset_command(struct kgi_graphic *state,kgi_uint cmd,kgi_uint arg)
{
	DEBUG1("kgim_chipset_set_mode: void\n");
	
	return EOK;
}
