/* $Id: rgb52x.h,v 1.5 1998/11/02 04:38:54 brian Exp $
******************************************************************************

   IBM RGB52x general ramdac register definitions

   Copyright (C) 1997		Torgeir Veimo

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

/*
**	This code was derived from the following source of information:
**
**	[1]	XFree86 X server source code
**	[2]	IBM RGB526(DB) Databook (Preliminary).
**	[3]	IBM RGB 524-526-624 Palette DAC Conversion - Application Note
**	[4]	Torgeir Veimo's original work
*/

#ifndef _IBM_RGBXXX_H
#define _IBM_RGBXXX_H

#include "IBM/vga_dac.h"

#define RGB526				0
#define RGB524 				1
#define RGB526_170 			0
#define RGB526_220 			1
#define RGB524_170 			2
#define RGB524_220 			3
#define DAC_UNKNOWN 			-1


/*
**	Extended registers
*/

#define DAC_EXT_LOW			0x04
#define DAC_EXT_HIGH			0x05
#define DAC_EXT_DATA			0x06
#define DAC_EXT_CTRL			0x07
	#define _RGB52X_EXT_AUTOINC_ENABLE	(1 << 0)
	#define _RGB52X_EXT_AUTOINC_DISABLE	(0 << 0)

/* functions for all IBM RGB ramdacs, to access extended registers*/

static inline void EXT_Address(struct kgi_display *dpy,kgiu16 reg){
	DAC_out8(dpy, (reg & 0x0FF) >> 0, DAC_EXT_LOW);
	DAC_out8(dpy, (reg & 0x700) >> 8, DAC_EXT_HIGH);
}

static inline void EXT_out8(struct kgi_display *dpy, kgiu8 val, kgiu16 reg)
{
	EXT_Address(dpy,reg);
	DAC_out8(dpy, val, DAC_EXT_DATA);
}

static inline kgiu8 EXT_in8(struct kgi_display *dpy, kgiu16 reg)
{
	EXT_Address(dpy,reg);
	return DAC_in8(dpy, DAC_EXT_DATA);
}

static inline void EXT_AutoIncrement_Enable(struct kgi_display *dpy){
	DAC_out8(dpy,
	_RGB52X_EXT_AUTOINC_ENABLE|(DAC_in8(dpy,DAC_EXT_CTRL)),
	 DAC_EXT_CTRL);
}

static inline void EXT_AutoIncrement_Disable(struct kgi_display *dpy){
	DAC_out8(dpy,
	(~_RGB52X_EXT_AUTOINC_ENABLE)&(DAC_in8(dpy,DAC_EXT_CTRL)), 
	DAC_EXT_CTRL);
}


/*
**	Internal registers. Accessed through the extended registers
*/

/*	Revision Level, preferred values.
**
**	The RGB524 has two different revision levels:
**
**	IBM37RGB524CF 17/22-A: SCLK INVT operate correctly: rev. level 0xE0 
**	IBM37RGB524CF 17/22: SCLK INVT doesn't operate correctly: rev.lvl 0xF0
**
**	The other ramdacs SCLK INVT operates correctly.
*/

#define DAC_REV					0x00
	#define _RGB524_EXT00_REV_LEVEL			0xE0
	#define _RGB524_EXT00_REV_BAD			0xF0 
	#define _RGB525_EXT00_REV_LEVEL			0x80
	#define _RGB526_EXT00_REV_LEVEL			0xC0
	#define _RGB624_EXT00_REV_LEVEL			0xC0

/*	Product ID
**
**	The RGB624 has a different product id to identify that YUV pixel 
**	formats are available.
*/

#define DAC_ID					0x01
	#define _RGB524_EXT01_PROD_ID			0x02
	#define _RGB525_EXT01_PROD_ID			0x02
	#define _RGB526_EXT01_PROD_ID			0x02
	#define _RGB624_EXT01_PROD_ID			0x30
#define DAC_MISC_CLOCK 		   		0x02
	#define _RGB52X_EXT02_DDOTCLK_ENABLE		(0 << 7)
	#define _RGB52X_EXT02_DDOTCLK_DISABLE		(1 << 7)
	#define _RGB52X_EXT02_SCLK_ENABLE		(0 << 6)
	#define _RGB52X_EXT02_SCLK_DISABLE		(1 << 6)
	#define _RGB52X_EXT02_B24P_DDOT_DIV_PLL		(0 << 5)
	#define _RGB52X_EXT02_B24P_DDOT_SCLK		(1 << 5)
	#define _RGB52X_EXT02_SCLK_INVT_DISABLE		(0 << 4)
	#define _RGB52X_EXT02_SCLK_INVT_ENABLE		(1 << 4)
	#define _RGB52X_EXT02_DDOT_PLL_DIV_1		(0 << 1)
	#define _RGB52X_EXT02_DDOT_PLL_DIV_2		(1 << 1)
	#define _RGB52X_EXT02_DDOT_PLL_DIV_4		(2 << 1)
	#define _RGB52X_EXT02_DDOT_PLL_DIV_8		(3 << 1)
	#define _RGB52X_EXT02_DDOT_PLL_DIV_16		(4 << 1)
	#define _RGB52X_EXT02_PLL_DISABLE		(0 << 0)
	#define _RGB52X_EXT02_PLL_ENABLE		(1 << 0)

/*
**	Sync Control
*/
#define DAC_SYNC				0x03
	#define _RGB52X_EXT03_DLY_SYNC_ADD		(0 << 7)
	#define _RGB52X_EXT03_DLY_SYNC_NOADD		(1 << 7)
	#define _RGB52X_EXT03_CSYN_INVT_DISABLE		(0 << 6)
	#define _RGB52X_EXT03_CSYN_INVT_ENABLE		(1 << 6)
	#define _RGB52X_EXT03_VSYN_INVT_DISABLE		(0 << 5)
	#define _RGB52X_EXT03_VSYN_INVT_ENABLE		(1 << 5)
	#define _RGB52X_EXT03_HSYN_INVT_DISABLE		(0 << 4)
	#define _RGB52X_EXT03_HSYN_INVT_ENABLE		(1 << 4)
	#define _RGB52X_EXT03_VSYN_CTRL_NORMAL		(0 << 2)
	#define _RGB52X_EXT03_VSYN_CTRL_HIGH		(1 << 2)
	#define _RGB52X_EXT03_VSYN_CTRL_LOW		(2 << 2)
	#define _RGB52X_EXT03_VSYN_VTRL_DISABLE		(3 << 2)
	#define _RGB52X_EXT03_HSYN_CTRL_NORMAL		(0 << 0)
	#define _RGB52X_EXT03_HSYN_CTRL_HIGH		(1 << 0)
	#define _RGB52X_EXT03_HSYN_CTRL_LOW		(2 << 0)
	#define _RGB52X_EXT03_HSYN_CTRL_DISABLE		(3 << 0)

/*
**	HSync Control
*/
#define DAC_HSYNC_POS				0x04
	#define _RGB52X_EXT04_HSYN_POS(n)		((n) & 0xF)

/*
**	Power Managment
*/
#define DAC_PWR_MGMT				0x05
	#define _RGB52X_EXT05_SCLK_PWR_NORMAL		(0 << 4)
	#define _RGB52X_EXT05_SCLK_PWR_DISABLE		(1 << 4)
	#define _RGB52X_EXT05_DDOT_PWR_NORMAL		(0 << 3)
	#define _RGB52X_EXT05_DDOT_PWR_DISABLE		(1 << 3)
	#define _RGB52X_EXT05_SYNC_PWR_NORMAL		(0 << 2)
	#define _RGB52X_EXT05_SYNC_PWR_DISABLE		(1 << 2)
	#define _RGB52X_EXT05_ICLK_PWR_NORMAL		(0 << 1)
	#define _RGB52X_EXT05_ICLK_PWR_DISABLE		(1 << 1)
	#define _RGB52X_EXT05_DAC_PWR_NORMAL		(0 << 0)
	#define _RGB52X_EXT05_DAC_PWR_DISABLE		(1 << 0)

/*
**	DAC Operation
*/
#define DAC_DAC_OP				0x06
	#define _RGB52X_EXT06_SOG_DISABLE		(0 << 3)
	#define _RGB52X_EXT06_SOG_ENABLE		(1 << 3)
	#define _RGB52X_EXT06_BRB_DISABLE		(0 << 2)
	#define _RGB52X_EXT06_BRB_ENABLE		(1 << 2)
	#define _RGB52X_EXT06_SLEW_SLOW			(0 << 1)
	#define _RGB52X_EXT06_SLEW_FAST			(1 << 1)
	#define _RGB52X_EXT06_DPE_DISABLE		(0 << 0)
	#define _RGB52X_EXT06_DPE_ENABLE		(1 << 0)

/*
**	Palette Control
*/
#define DAC_PAL					0x07
	#define _RGB52X_EXT07_6BIT_LINEAR_ENABLE	(0 << 7)
	#define _RGB52X_EXT07_6BIT_LINEAR_DISABLE	(1 << 7)
	#define _RGB52X_EXT07_PALETTE_PARTITION(n)	((n) & 0xF)

#define DAC_SYSCLK				0x08	/* not RGB525 */
	#define _RGB52X_EXT08_SYSCLK_ENABLE		(0 << 6)
	#define _RGB52X_EXT08_SYSCLK_DISABLE		(1 << 6)
	#define _RGB52X_EXT08_SYSCLK_SRC_PLL		(0 << 1)
	#define _RGB52X_EXT08_SYSCLK_SRC_REFCLK		(1 << 1)
	#define _RGB52X_EXT08_SYSCLK_PROG_DISABLE	(0 << 0)
	#define _RGB52X_EXT08_SYSCLK_PROG_ENABLE	(1 << 0)
/*
**	Pixel Representation
*/
#define DAC_PIX_FMT				0x0a
	#define _RGB52X_EXT0A_PIXEL_FORMAT_4BPP		(2 << 0)
	#define _RGB52X_EXT0A_PIXEL_FORMAT_8BPP		(3 << 0)
	#define _RGB52X_EXT0A_PIXEL_FORMAT_16BPP	(4 << 0)
	#define _RGB52X_EXT0A_PIXEL_FORMAT_24BPP	(5 << 0)
	#define _RGB52X_EXT0A_PIXEL_FORMAT_32BPP	(6 << 0)

/*
**	8 Bit Pixel Control
*/
#define DAC_8BPP_CTRL				0x0b
	#define _RGB52X_EXT0B_B8_DCOL_INDIRECT		(0 << 0)
	#define _RGB52X_EXT0B_B8_DCOL_DIRECT		(1 << 0)

/*
**	16 Bit Pixel Control
*/#define DAC_16BPP_CTRL			0x0c
	#define _RGB52X_EXT0C_B16_DCOL_INDIRECT		(0 << 6)
	#define _RGB52X_EXT0C_B16_DCOL_DYNAMIC		(1 << 6)
	#define _RGB52X_EXT0C_B16_DCOL_DIRECT		(3 << 6)
	#define _RGB52X_EXT0C_B16_POL_SET_LOOKUP	(0 << 5)
	#define _RGB52X_EXT0C_B16_POL_SET_BYPASS	(1 << 5)
	#define _RGB52X_EXT0C_B16_ZIB			(0 << 2)
	#define _RGB52X_EXT0C_B16_LINEAR		(1 << 2)
	#define _RGB52X_EXT0C_B16_555			(0 << 1)
	#define _RGB52X_EXT0C_B16_565			(1 << 1)
	#define _RGB52X_EXT0C_B16_SPARSE		(0 << 0)
	#define _RGB52X_EXT0C_B16_CONTIGUOUS		(1 << 0)
/*
**	24 Bit Pixel Contol
*/
#define DAC_24BPP_CTRL				0x0d
	#define _RGB52X_EXT0D_B24_DCOL_INDIRECT		(0 << 0)
	#define _RGB52X_EXT0D_B24_DCOL_DIRECT		(1 << 0)

/*
**	32 Bit Pixel Control
*/
#define DAC_32BPP_CTRL				0x0e
	#define _RGB52X_EXT0E_B32_SET_FORCES_LOOKUP	(0 << 2)
	#define _RGB52X_EXT0E_B32_SET_FORCES_BYPASS	(1 << 2)
	#define _RGB52X_EXT0E_B32_DCOL_INDIRECT		(0 << 0)
	#define _RGB52X_EXT0E_B32_DCOL_DYNAMIC		(1 << 0)
	#define _RGB52X_EXT0E_B32_DCOL_DIRECT		(3 << 0)


/*
**	Frequency control
*/

/*
**	PLL Control One
*/
#define DAC_PLL_CTRL1				0x10
	#define _RGB52X_EXT10_REF_SRC_REFCLK		(0 << 4)
	#define _RGB52X_EXT10_REF_SRC_EXTCLOCK		(1 << 4)
	#define _RGB52X_EXT10_PLL_EXT_FS_DIRECT		(0 << 0)
	#define _RGB52X_EXT10_PLL_EXT_FS_M_N		(1 << 0)
	#define _RGB52X_EXT10_PLL_INT_FS_DIRECT		(2 << 0)
	#define _RGB52X_EXT10_PLL_INT_FS_M_N		(3 << 0)

/*
**	PLL Control Two
*/
#define DAC_PLL_CTRL2				0x11
	#define _RGB52X_EXT11_PLL_INT_FS(n)		((n) & 0xF)


/*
**	PLL Ref Div Count
*/
#define DAC_PLL_REF_DIV_FIX			0x14
	#define _RGB52X_EXT14_REF_DIV_COUNT		((n) & 0xF)

	#define _RGB52X_EXT14_PLL_REFCLK_4_MHz       	(0x02)
	#define _RGB52X_EXT14_PLL_REFCLK_6_MHz       	(0x03)
	#define _RGB52X_EXT14_PLL_REFCLK_8_MHz       	(0x04)
	#define _RGB52X_EXT14_PLL_REFCLK_10_MHz      	(0x05)
	#define _RGB52X_EXT14_PLL_REFCLK_12_MHz      	(0x06)
	#define _RGB52X_EXT14_PLL_REFCLK_14_MHz      	(0x07)
	#define _RGB52X_EXT14_PLL_REFCLK_16_MHz      	(0x08)
	#define _RGB52X_EXT14_PLL_REFCLK_18_MHz     	(0x09)
	#define _RGB52X_EXT14_PLL_REFCLK_20_MHz      	(0x0A)
	#define _RGB52X_EXT14_PLL_REFCLK_22_MHz      	(0x0B)
	#define _RGB52X_EXT14_PLL_REFCLK_24_MHz      	(0x0C)
	#define _RGB52X_EXT14_PLL_REFCLK_26_MHz      	(0x0D)
	#define _RGB52X_EXT14_PLL_REFCLK_28_MHz      	(0x0E)
	#define _RGB52X_EXT14_PLL_REFCLK_30_MHz     	(0x0F)
	#define _RGB52X_EXT14_PLL_REFCLK_32_MHz      	(0x10)
	#define _RGB52X_EXT14_PLL_REFCLK_34_MHz      	(0x11)
	#define _RGB52X_EXT14_PLL_REFCLK_36_MHz      	(0x12)
	#define _RGB52X_EXT14_PLL_REFCLK_38_MHz      	(0x13)
	#define _RGB52X_EXT14_PLL_REFCLK_40_MHz      	(0x14)
	#define _RGB52X_EXT14_PLL_REFCLK_42_MHz      	(0x15)
	#define _RGB52X_EXT14_PLL_REFCLK_44_MHz      	(0x16)
	#define _RGB52X_EXT14_PLL_REFCLK_46_MHz      	(0x17)
	#define _RGB52X_EXT14_PLL_REFCLK_48_MHz      	(0x18)
	#define _RGB52X_EXT14_PLL_REFCLK_50_MHz      	(0x19)
	#define _RGB52X_EXT14_PLL_REFCLK_52_MHz      	(0x1A)
	#define _RGB52X_EXT14_PLL_REFCLK_54_MHz      	(0x1B)
	#define _RGB52X_EXT14_PLL_REFCLK_56_MHz      	(0x1C)
	#define _RGB52X_EXT14_PLL_REFCLK_58_MHz      	(0x1D)
	#define _RGB52X_EXT14_PLL_REFCLK_60_MHz      	(0x1E)
	#define _RGB52X_EXT14_PLL_REFCLK_62_MHz      	(0x1F)

#define DAC_SYSCLK_REF_DIV			0x15	/* not RGB525 */
#define DAC_SYSCLK_VCO_DIV			0x16	/* not RGB525 */

/*
**	F0-F15[7:0]
*/
#define DAC_F0					0x20
#define DAC_F1					0x21
#define DAC_F2					0x22
#define DAC_F3					0x23
#define DAC_F4					0x24
#define DAC_F5					0x25
#define DAC_F6					0x26
#define DAC_F7					0x27
#define DAC_F8					0x28
#define DAC_F9					0x29
#define DAC_F10					0x2A
#define DAC_F11					0x2B
#define DAC_F12					0x2C
#define DAC_F13					0x2D
#define DAC_F14					0x2E
#define DAC_F15					0x2F

	#define _RGB52X_EXTXX_DF(n)			(((n) & 0x3) << 6)
	#define _RGB52X_EXTXX_VCO_DIV_COUNT(n)		((n) & 0x3F)

#define DAC_M0					0x20
#define DAC_M1					0x22
#define DAC_M2					0x24
#define DAC_M3					0x26
#define DAC_M4					0x28
#define DAC_M5					0x2A
#define DAC_M6					0x2C
#define DAC_M7					0x2E

#define DAC_N0					0x21
#define DAC_N1					0x23
#define DAC_N2					0x25
#define DAC_N3					0x27
#define DAC_N4 					0x29
#define DAC_N5  				0x2B
#define DAC_N6					0x2D
#define DAC_N7					0x2F


#define DAC_CURS_CTRL				0x30
	#define _RGB52X_EXT30_SMLC_PART0		(0 << 6)
	#define _RGB52X_EXT30_SMLC_PART1		(1 << 6)
	#define _RGB52X_EXT30_SMLC_PART2		(2 << 6)
	#define _RGB52X_EXT30_SMLC_PART3		(3 << 6)
	#define _RGB52X_EXT30_CURS_ORDER_R_TO_L		(0 << 5)
	#define _RGB52X_EXT30_CURS_ORDER_L_TO_R		(1 << 5)
	#define _RGB52X_EXT30_LOC_READ_LAST_WRITTEN	(0 << 4)
	#define _RGB52X_EXT30_LOC_READ_ACTUAL		(1 << 4)
	#define _RGB52X_EXT30_UPDT_CTRL_DELAYED		(0 << 3)
	#define _RGB52X_EXT30_UPDT_CTRL_IMMEDIATE	(1 << 3)
	#define _RGB52X_EXT30_CURSOR_SZE_32		(0 << 2)
	#define _RGB52X_EXT30_CURSOR_SIZE_64		(1 << 2)
	#define _RGB52X_EXT30_CURSOR_MODE_OFF		(0 << 0)
	#define _RGB52X_EXT30_CURSOR_MODE_3COLOR	(1 << 0)
	#define _RGB52X_EXT30_CURSOR_MODE_XGA		(2 << 0)
	#define _RGB52X_EXT30_CURSOR_MODE_X11		(3 << 0)



#define DAC_CURS_XLOW				0x31
#define DAC_CURS_XHIGH				0x32
#define DAC_CURS_YLOW				0x33
#define DAC_CURS_YHIGH				0x34
#define DAC_CURS_HOTSPOT_X   		  	0x35
#define DAC_CURS_HOTSPOT_Y   		  	0x36

#define DAC_CURS_COLSTART			0x40
#define DAC_CURS_COL1_R				0x40
#define DAC_CURS_COL1_G				0x41
#define DAC_CURS_COL1_B				0x42
#define DAC_CURS_COL2_R				0x43
#define DAC_CURS_COL2_G				0x44
#define DAC_CURS_COL2_B				0x45
#define DAC_CURS_COL3_R				0x46
#define DAC_CURS_COL3_G				0x47
#define DAC_CURS_COL3_B				0x48

#define DAC_BORDER_COL_R			0x60
#define DAC_BORDER_COL_G			0x61
#define DAC_BORDER_COL_B			0x62

/*
**	Misc Control One
*/
#define DAC_MISC_CTRL_1				0x70
	#define _RGB52X_EXT70_MISR_CTRL_OFF		(0 << 7)
	#define _RGB52X_EXT70_MISR_CTRL_ON		(1 << 7)
	#define _RGB52X_EXT70_VMSK_CTRL_OFF		(0 << 6)
	#define _RGB52X_EXT70_VMSK_CTRL_ON		(1 << 6)
	#define _RGB52X_EXT70_PADR_RFMT_READ_ADDR    	(0 << 5)
	#define _RGB52X_EXT70_PADR_RFMT_PAL_STATE    	(1 << 5)
	#define _RGB52X_EXT70_SENS_DSAB_ENABLE       	(0 << 4)
	#define _RGB52X_EXT70_SENS_DSAB_DISABLE      	(1 << 4)
	#define _RGB52X_EXT70_SENS_SEL_BIT3		(0 << 3)
	#define _RGB52X_EXT70_SENS_SEL_BIT7		(1 << 3)
	#define _RGB52X_EXT70_VRAM_SIZE_32		(0 << 0)
	#define _RGB52X_EXT70_VRAM_SIZE_64		(1 << 0)

/*
**	Misc Control Two
*/
#define DAC_MISC_CTRL_2				0x71
	#define _RGB52X_EXT71_PCLK_SEL_LCLK		(0 << 6)
	#define _RGB52X_EXT71_PCLK_SEL_PLL		(1 << 6)
	#define _RGB52X_EXT71_PCLK_SEL_EXT		(2 << 6)
	#define _RGB52X_EXT71_PCLK_SEL_MASK		(3 << 6)
	#define _RGB52X_EXT71_INTL_MODE_DISABLE		(0 << 5)
	#define _RGB52X_EXT71_INTL_MODE_ENABLE		(1 << 5)
	#define _RGB52X_EXT71_BLANK_CTRL_NORMAL		(0 << 4)
	#define _RGB52X_EXT71_BLANK_CTRL_BLANKED	(1 << 4)
	#define _RGB52X_EXT71_COL_RES_6BIT		(0 << 2)
	#define _RGB52X_EXT71_COL_RES_8BIT		(1 << 2)
	#define _RGB52X_EXT71_BRDR_INTL_BRDR		(0 << 1)
	#define _RGB52X_EXT71_BRDR_INTL_INTL		(1 << 1)
	#define _RGB52X_EXT71_PORT_SEL_VGA		(0 << 0)
	#define _RGB52X_EXT71_PORT_SEL_VRAM		(1 << 0)

/*
**	Misc Control Three
*/
#define DAC_MISC_CTRL_3				0x72
	#define _RGB52X_EXT72_SWAP_RB_DISABLE		(0 << 7)
	#define _RGB52X_EXT72_SWAP_RB_ENABLE		(1 << 7)
	#define _RGB52X_EXT72_SWAP_WORD_31_00_FIRST	(0 << 4)
	#define _RGB52X_EXT72_SWAP_WORD_63_32_FIRST	(1 << 4)
	#define _RGB52X_EXT72_SWAP_NIB_07_04_FIRST	(0 << 1)
	#define _RGB52X_EXT72_SWAP_NIB_03_00_FIRST	(1 << 1)
/*
**	526-only registers
*/

#define DAC_KEY_CTRL				0x78
	#define _RGB526_EXT78_KEY_ENABLE		(1 << 0)
	#define _RGB526_EXT78_KEY_DISABLE		(0 << 0)

/*
**	Misc Diagnostic Support
*/

#define DAC_DAC_SENSE				0x82
#define DAC_MISR_R				0x84
#define DAC_MISR_G				0x86
#define DAC_MISR_B				0x88
#define DAC_PLL_VCO_DIV_IN			0x8e
#define DAC_PLL_REF_DIV_IN			0x8f
#define DAC_VRAM_MASK_0				0x90
#define DAC_VRAM_MASK_1				0x91
#define DAC_VRAM_MASK_2				0x92
#define DAC_VRAM_MASK_3				0x93

#define DAC_CURS_ARRAY				0x100

/*
**	Timing Requirements
*/
#define DAC_SYSCLK_LOW 			16.25
#define DAC_SYSCLK_HIGH			100

#define DAC_DDOTDIV_HIGH		100

/*
**  	State Array used by driver
*/
#define ARRAY_MISC_CLOCK 		0
#define ARRAY_SYNC			1
#define ARRAY_HSYNC_POS			2
#define ARRAY_PWR_MGMT			3
#define ARRAY_DAC_OP			4
#define ARRAY_DAC_PAL			5
#define ARRAY_DAC_SYSCLK		6
#define ARRAY_DAC_PIX_FMT		7
#define ARRAY_8BPP_CTRL			8
#define ARRAY_16BPP_CTRL		9
#define ARRAY_24BPP_CTRL		10
#define ARRAY_32BPP_CTRL		11
#define ARRAY_MISC_CTRL1		12
#define ARRAY_MISC_CTRL2		13
#define ARRAY_MISC_CTRL3		14

#endif /* _IBM_RGBXXX_H */

